@@ -122,6 +122,7 @@ na_tray_applet_style_updated (GtkWidget *widget)
   GdkColor         warning;
   GdkColor         success;
   gint             padding;
+  gint             icon_size;
 
   GTK_WIDGET_CLASS (na_tray_applet_parent_class)->style_updated (widget);
 
@@ -145,8 +146,10 @@ na_tray_applet_style_updated (GtkWidget *widget)
   na_tray_set_colors (applet->priv->tray, &fg, &error, &warning, &success);
 
   gtk_widget_style_get (widget, "icon-padding", &padding, NULL);
-
   na_tray_set_padding (applet->priv->tray, padding);
+
+  gtk_widget_style_get (widget, "icon-size", &icon_size, NULL);
+  na_tray_set_icon_size (applet->priv->tray, icon_size);
 }
 
 static void
@@ -212,6 +215,14 @@ na_tray_applet_class_init (NaTrayAppletClass *class)
                             0, G_MAXINT, 0,
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  gtk_widget_class_install_style_property (
+          widget_class,
+          g_param_spec_int ("icon-size",
+                            "Icon size",
+                            "If non-zero, hardcodes the size of the icons in pixels",
+                            0, G_MAXINT, 0,
+                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   g_type_class_add_private (class, sizeof (NaTrayAppletPrivate));
 }
 
@@ -260,13 +271,11 @@ applet_factory (PanelApplet *applet,
 #ifdef NOTIFICATION_AREA_INPROCESS
 PANEL_APPLET_IN_PROCESS_FACTORY ("NotificationAreaAppletFactory",
 				 NA_TYPE_TRAY_APPLET,
-				 "NotificationArea",
 				 applet_factory,
 				 NULL)
 #else
 PANEL_APPLET_OUT_PROCESS_FACTORY ("NotificationAreaAppletFactory",
 				  NA_TYPE_TRAY_APPLET,
-				  "NotificationArea",
 				  applet_factory,
 				  NULL)
 #endif
