@@ -40,6 +40,10 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
+#ifdef HAVE_TELEPATHY_GLIB
+#include <telepathy-glib/account-manager.h>
+#endif
+
 #include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-keyfile.h>
@@ -95,8 +99,96 @@ struct _PanelPlaceMenuItemPrivate {
 struct _PanelDesktopMenuItemPrivate {
 	GtkWidget   *menu;
 	PanelWidget *panel;
+
+#ifdef HAVE_TELEPATHY_GLIB
+	GList            *presence_items;
+	guint             presence_changed_id;
+	TpAccountManager *account_manager;
+#endif
 };
 
+#ifdef HAVE_TELEPATHY_GLIB
+static void
+panel_menu_item_activate_presence (GtkWidget        *menuitem,
+				   TpAccountManager *account_manager)
+{
+	PanelSessionManagerPresenceType  presence_type;
+	TpConnectionPresenceType         tp_presence_type;
+	const char                      *status;
+	char                            *message;
+
+	presence_type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem),
+							    "panel-menu-presence"));
+
+	panel_session_manager_set_presence (panel_session_manager_get (),
+					    presence_type);
+
+	tp_presence_type = tp_account_manager_get_most_available_presence (account_manager,
+									   NULL,
+									   &message);
+
+	if (tp_presence_type == TP_CONNECTION_PRESENCE_TYPE_UNSET ||
+	    tp_presence_type == TP_CONNECTION_PRESENCE_TYPE_OFFLINE ||
+	    tp_presence_type == TP_CONNECTION_PRESENCE_TYPE_UNKNOWN ||
+	    tp_presence_type == TP_CONNECTION_PRESENCE_TYPE_ERROR)
+		goto free_message;
+
+	if (presence_type == PANEL_SESSION_MANAGER_PRESENCE_AVAILABLE) {
+		tp_presence_type = TP_CONNECTION_PRESENCE_TYPE_AVAILABLE;
+		status = "available";
+	} else if (presence_type == PANEL_SESSION_MANAGER_PRESENCE_BUSY) {
+		tp_presence_type = TP_CONNECTION_PRESENCE_TYPE_BUSY;
+		status = "busy";
+	} else
+		goto free_message;
+
+	tp_account_manager_set_all_requested_presences (account_manager,
+							tp_presence_type,
+							status, message);
+
+free_message:
+	g_free (message);
+}
+
+static GtkWidget *
+panel_menu_item_presence_new (TpAccountManager                *account_manager,
+			      PanelSessionManagerPresenceType  presence_type,
+			      const char                      *name,
+			      const char                      *icon,
+			      gboolean                         use_icon)
+{
+	GtkWidget *item;
+
+	if (!account_manager)
+		return NULL;
+
+	item = gtk_check_menu_item_new ();
+	setup_menuitem (item, GTK_ICON_SIZE_INVALID, NULL, name);
+	gtk_check_menu_item_set_draw_as_radio (GTK_CHECK_MENU_ITEM (item), TRUE);
+
+	/* TODO: we need to add an icon at the right of this CheckMenuItem */
+#if 0
+	if (use_icon) {
+		GtkWidget *image;
+		image = gtk_image_new_from_icon_name (icon,
+						      panel_menu_icon_get_size ());
+		gtk_container_add (GTK_CONTAINER (item), image);
+	}
+#endif
+
+	g_object_set_data (G_OBJECT (item), "panel-menu-presence",
+			   GINT_TO_POINTER (presence_type));
+
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (panel_menu_item_activate_presence),
+			  account_manager);
+	g_signal_connect (G_OBJECT (item), "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+
+	return item;
+}
+#endif
+
 static void
 activate_uri_on_screen (const char *uri,
 			GdkScreen  *screen)
@@ -346,6 +438,65 @@ panel_menu_items_create_action_item (PanelActionButtonType action_type)
 							 NULL, NULL, FALSE);
 }
 
+#define GDM_FLEXISERVER_COMMAND "gdmflexiserver"
+#define GDM_FLEXISERVER_ARGS    "--startnew"
+
+static void
+panel_menu_item_activate_switch_user (GtkWidget *menuitem,
+				      gpointer   user_data)
+{
+	GdkScreen *screen;
+	GAppInfo  *app_info;
+
+	if (panel_lockdown_get_disable_switch_user_s ())
+		return;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (menuitem));
+	app_info = g_app_info_create_from_commandline (GDM_FLEXISERVER_COMMAND " " GDM_FLEXISERVER_ARGS,
+						       GDM_FLEXISERVER_COMMAND,
+						       G_APP_INFO_CREATE_NONE,
+						       NULL);
+
+	if (app_info) {
+		GdkAppLaunchContext *launch_context;
+		GdkDisplay          *display;
+
+		display = gdk_screen_get_display (screen);
+		launch_context = gdk_display_get_app_launch_context (display);
+		gdk_app_launch_context_set_screen (launch_context, screen);
+
+		g_app_info_launch (app_info, NULL,
+				   G_APP_LAUNCH_CONTEXT (launch_context),
+				   NULL);
+
+		g_object_unref (launch_context);
+		g_object_unref (app_info);
+	}
+}
+
+static GtkWidget *
+panel_menu_items_create_switch_user (gboolean use_icon)
+{
+	GtkWidget *item;
+
+	if (use_icon) {
+		item = panel_image_menu_item_new ();
+        } else {
+		item = gtk_image_menu_item_new ();
+	}
+
+	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
+				   NULL, NULL, NULL, _("Switch User"));
+
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (panel_menu_item_activate_switch_user),
+			  NULL);
+	g_signal_connect (G_OBJECT (item), "button_press_event",
+			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+
+	return item;
+}
+
 static void
 panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 {
@@ -1192,15 +1343,47 @@ panel_desktop_menu_item_create_menu (PanelDesktopMenuItem *desktop_item,
 {
 	GtkWidget *desktop_menu;
 	GtkWidget *item;
+#ifdef HAVE_TELEPATHY_GLIB
+	gboolean   added;
+#endif
 
 	desktop_menu = panel_create_menu ();
 
-	item = panel_menu_item_desktop_new ("gnome-control-center.desktop",
+#ifdef HAVE_TELEPATHY_GLIB
+	desktop_item->priv->account_manager = tp_account_manager_dup ();
+
+	item = panel_menu_item_presence_new (desktop_item->priv->account_manager,
+					     PANEL_SESSION_MANAGER_PRESENCE_AVAILABLE,
+					     _("Available"),
+					     PANEL_ICON_USER_AVAILABLE, TRUE);
+	if (item) {
+		desktop_item->priv->presence_items = g_list_prepend (desktop_item->priv->presence_items, item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (desktop_menu), item);
+		added = TRUE;
+	}
+
+	item = panel_menu_item_presence_new (desktop_item->priv->account_manager,
+					     PANEL_SESSION_MANAGER_PRESENCE_BUSY,
+					     _("Busy"),
+					     PANEL_ICON_USER_BUSY, TRUE);
+	if (item) {
+		desktop_item->priv->presence_items = g_list_prepend (desktop_item->priv->presence_items, item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (desktop_menu), item);
+		added = TRUE;
+	}
+
+	if (added)
+		add_menu_separator (desktop_menu);
+#endif
+
+	item = panel_menu_item_desktop_new ("gnome-online-accounts-panel.desktop",
 					    NULL, FALSE);
 	if (item)
 		gtk_menu_shell_append (GTK_MENU_SHELL (desktop_menu), item);
 
-	item = panel_menu_item_desktop_new ("yelp.desktop",
+	/* Do not force the string like in gnome-shell, but just use the one
+	 * from the .desktop file */
+	item = panel_menu_item_desktop_new ("gnome-control-center.desktop",
 					    NULL, FALSE);
 	if (item)
 		gtk_menu_shell_append (GTK_MENU_SHELL (desktop_menu), item);
@@ -1281,6 +1464,28 @@ panel_place_menu_item_finalize (GObject *object)
 	G_OBJECT_CLASS (panel_place_menu_item_parent_class)->finalize (object);
 }
 
+static void
+panel_desktop_menu_item_finalize (GObject *object)
+{
+	PanelDesktopMenuItem *menuitem = (PanelDesktopMenuItem *) object;
+
+#ifdef HAVE_TELEPATHY_GLIB
+	g_list_free (menuitem->priv->presence_items);
+	menuitem->priv->presence_items = NULL;
+
+	if (menuitem->priv->presence_changed_id != 0)
+		g_signal_handler_disconnect (panel_session_manager_get (),
+					     menuitem->priv->presence_changed_id);
+	menuitem->priv->presence_changed_id = 0;
+
+	if (menuitem->priv->account_manager != NULL)
+		g_object_unref (menuitem->priv->account_manager);
+	menuitem->priv->account_manager = NULL;
+#endif
+
+	G_OBJECT_CLASS (panel_desktop_menu_item_parent_class)->finalize (object);
+}
+
 static void
 panel_place_menu_item_init (PanelPlaceMenuItem *menuitem)
 {
@@ -1380,14 +1585,20 @@ static void
 panel_desktop_menu_item_init (PanelDesktopMenuItem *menuitem)
 {
 	menuitem->priv = PANEL_DESKTOP_MENU_ITEM_GET_PRIVATE (menuitem);
+
+#ifdef HAVE_TELEPATHY_GLIB
+	menuitem->priv->presence_items = NULL;
+	menuitem->priv->presence_changed_id = 0;
+	menuitem->priv->account_manager = NULL;
+#endif
 }
 
 static void
 panel_place_menu_item_class_init (PanelPlaceMenuItemClass *klass)
 {
-	GObjectClass *gobject_class = (GObjectClass   *) klass;
+	GObjectClass *gobject_class = (GObjectClass *) klass;
 
-	gobject_class->finalize  = panel_place_menu_item_finalize;
+	gobject_class->finalize = panel_place_menu_item_finalize;
 
 	g_type_class_add_private (klass, sizeof (PanelPlaceMenuItemPrivate));
 }
@@ -1395,6 +1606,10 @@ panel_place_menu_item_class_init (PanelPlaceMenuItemClass *klass)
 static void
 panel_desktop_menu_item_class_init (PanelDesktopMenuItemClass *klass)
 {
+	GObjectClass *gobject_class = (GObjectClass *) klass;
+
+	gobject_class->finalize = panel_desktop_menu_item_finalize;
+
 	g_type_class_add_private (klass, sizeof (PanelDesktopMenuItemPrivate));
 }
 
@@ -1424,31 +1639,114 @@ panel_place_menu_item_new (gboolean use_image)
 	return GTK_WIDGET (menuitem);
 }
 
+#ifdef HAVE_TELEPATHY_GLIB
+static void
+panel_desktop_menu_item_on_presence_changed (PanelSessionManager             *manager,
+					     PanelSessionManagerPresenceType  presence_type,
+					     PanelDesktopMenuItem            *desktop_item)
+{
+	const char *icon;
+	GtkWidget  *image;
+	GList      *l;
+
+	switch (presence_type) {
+	case PANEL_SESSION_MANAGER_PRESENCE_AVAILABLE:
+		icon = PANEL_ICON_USER_AVAILABLE;
+		break;
+        case PANEL_SESSION_MANAGER_PRESENCE_INVISIBLE:
+		icon = PANEL_ICON_USER_INVISIBLE;
+		break;
+        case PANEL_SESSION_MANAGER_PRESENCE_BUSY:
+		icon = PANEL_ICON_USER_BUSY;
+		break;
+	case PANEL_SESSION_MANAGER_PRESENCE_IDLE:
+	default:
+		icon = PANEL_ICON_USER_IDLE;
+		break;
+	}
+
+	image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (desktop_item));
+	/* we only have an image if we are specifically using an icon for this
+	 * menu */
+	if (image) {
+		gtk_image_set_from_icon_name (GTK_IMAGE (image),
+					      icon, panel_menu_icon_get_size ());
+	}
+
+	for (l = desktop_item->priv->presence_items; l != NULL; l = l->next) {
+		PanelSessionManagerPresenceType for_presence;
+		GObject *object = l->data;
+
+		for_presence = GPOINTER_TO_INT (g_object_get_data (object,
+								   "panel-menu-presence"));
+		g_signal_handlers_block_by_func (object,
+						 panel_menu_item_activate_presence,
+						 desktop_item->priv->account_manager);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (object),
+						for_presence == presence_type);
+		g_signal_handlers_unblock_by_func (object,
+						   panel_menu_item_activate_presence,
+						   desktop_item->priv->account_manager);
+	}
+}
+#endif
+
 GtkWidget *
 panel_desktop_menu_item_new (gboolean use_image,
 			     gboolean append_lock_logout)
 {
 	PanelDesktopMenuItem *menuitem;
-	GtkWidget            *image;
+	char                 *name;
+#ifdef HAVE_TELEPATHY_GLIB
+	PanelSessionManager  *manager;
+#endif
 
 	menuitem = g_object_new (PANEL_TYPE_DESKTOP_MENU_ITEM, NULL);
 
-	if (use_image)
-		image = gtk_image_new_from_icon_name ("computer",
-						      panel_menu_icon_get_size ());
-	else
-		image = NULL;
+	name = panel_util_get_user_name ();
 
-	setup_menuitem (GTK_WIDGET (menuitem),
-			image ? panel_menu_icon_get_size () : GTK_ICON_SIZE_INVALID,
-			image,
-			_("System"));
+	if (use_image) {
+#ifdef HAVE_TELEPATHY_GLIB
+		setup_menu_item_with_icon (GTK_WIDGET (menuitem),
+					   panel_menu_icon_get_size (),
+					   PANEL_ICON_USER_AVAILABLE,
+					   NULL, NULL,
+					   name);
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menuitem),
+							   TRUE);
+#else
+		setup_menu_item_with_icon (GTK_WIDGET (menuitem),
+					   panel_menu_icon_get_size (),
+					   PANEL_ICON_COMPUTER,
+					   NULL, NULL,
+					   name);
+#endif
+	} else
+		setup_menuitem (GTK_WIDGET (menuitem),
+				GTK_ICON_SIZE_INVALID, NULL,
+				name);
+
+	g_free (name);
 
 	menuitem->priv->menu = panel_desktop_menu_item_create_menu (menuitem,
 								    append_lock_logout);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
 				   menuitem->priv->menu);
 
+#ifdef HAVE_TELEPATHY_GLIB
+	manager = panel_session_manager_get ();
+
+	menuitem->priv->presence_changed_id =
+		g_signal_connect (manager,
+				  "presence-changed",
+				  G_CALLBACK (panel_desktop_menu_item_on_presence_changed),
+				  menuitem);
+
+	panel_desktop_menu_item_on_presence_changed (manager,
+						     panel_session_manager_get_presence (manager),
+						     menuitem);
+#endif
+
 	return GTK_WIDGET (menuitem);
 }
 
@@ -1485,6 +1783,7 @@ panel_menu_items_lock_logout_separator_notified (PanelLockdown *lockdown,
 	GtkWidget *separator = user_data;
 
 	if (!panel_lockdown_get_disable_lock_screen (lockdown) ||
+	    !panel_lockdown_get_disable_switch_user (lockdown) ||
 	    !panel_lockdown_get_disable_log_out (lockdown))
 		gtk_widget_show (separator);
 	else
@@ -1497,9 +1796,6 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 	GList      *children;
 	GList      *last;
 	GtkWidget  *item;
-	const char *translate;
-	char       *label;
-	char       *tooltip;
 
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
 	last = g_list_last (children);
@@ -1529,38 +1825,19 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 					G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
 	}
 
-	/* Translators: translate "1" (msgctxt: "panel:showusername") to anything
-	 * but "1" if "Log Out %s" doesn't make any sense in your
-	 * language (where %s is a username).
-	 */
-	translate = C_("panel:showusername", "1");
-	if (strcmp (translate, "1") == 0) {
-		const char *user_name;
+	item = panel_menu_items_create_switch_user (FALSE);
 
-		user_name = g_get_real_name ();
-		if (!user_name || !user_name [0])
-			user_name = g_get_user_name ();
-
-		/* keep those strings in sync with the ones in
-		 * panel-action-button.c */
-		/* Translators: this string is used ONLY if you translated
-		 * "1" (msgctxt: "panel:showusername") to "1" */
-		label = g_strdup_printf (_("Log Out %s..."),
-					 g_get_user_name ());
-		/* Translators: this string is used ONLY if you translated
-		 * "1" (msgctxt: "panel:showusername") to "1" */
-		tooltip = g_strdup_printf (_("Log out %s of this session to "
-					     "log in as a different user"),
-					   user_name);
-	} else {
-		label   = NULL;
-		tooltip = NULL;
+	if (item != NULL) {
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_object_bind_property (panel_lockdown_get (),
+					"disable-switch-user",
+					item,
+					"visible",
+					G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
 	}
 
 	item = panel_menu_items_create_action_item_full (PANEL_ACTION_LOGOUT,
-							 label, tooltip, TRUE);
-	g_free (label);
-	g_free (tooltip);
+							 NULL, NULL, TRUE);
 
 	if (item != NULL) {
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1576,7 +1853,17 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 		item = panel_menu_items_create_action_item_full (PANEL_ACTION_SHUTDOWN,
 								 NULL, NULL, TRUE);
 		if (item != NULL) {
+			GtkWidget *sep;
+
+			sep = add_menu_separator (menu);
+
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_object_bind_property (panel_lockdown_get (),
+						"disable-log-out",
+						sep,
+						"visible",
+						G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
 			g_object_bind_property (panel_lockdown_get (),
 						"disable-log-out",
 						item,
